@@ -1,8 +1,5 @@
 package Catalyst::ActionContainer;
 
-use strict;
-use base qw/Class::Accessor::Fast/;
-
 =head1 NAME
 
 Catalyst::ActionContainer - Catalyst Action Container
@@ -18,25 +15,25 @@ to represent the various dispatch points in your application.
 
 =cut
 
-__PACKAGE__->mk_accessors(qw/part actions/);
+use Moose;
+with 'MooseX::Emulate::Class::Accessor::Fast';
+
+has part => (is => 'rw', required => 1);
+has actions => (is => 'rw', required => 1, lazy => 1, default => sub { {} });
+
+around BUILDARGS => sub {
+    my ($next, $self, @args) = @_;
+    unshift @args, 'part' if scalar @args == 1 && !ref $args[0];
+    return $self->$next(@args);
+};
+
+no Moose;
 
 use overload (
-
     # Stringify to path part for tree search
-    q{""} => sub { shift->{part} },
-
+    q{""} => sub { shift->part },
 );
 
-sub new {
-    my ( $class, $fields ) = @_;
-
-    $fields = { part => $fields, actions => {} } unless ref $fields;
-
-    $class->SUPER::new($fields);
-}
-
-
-
 sub get_action {
     my ( $self, $name ) = @_;
     return $self->actions->{$name} if defined $self->actions->{$name};
@@ -49,6 +46,8 @@ sub add_action {
     $self->actions->{$name} = $action;
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -78,9 +77,13 @@ Accessor to the actions hashref, containing all actions in this container.
 Accessor to the path part this container resolves to. Also what the container
 stringifies to.
 
-=head1 AUTHOR
+=head2 meta
 
-Matt S. Trout 
+Provided by Moose
+
+=head1 AUTHORS
+
+Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
