@@ -2,8 +2,7 @@ package Catalyst::ClassData;
 
 use Moose::Role;
 use Class::MOP;
-use Class::MOP::Object;
-use Scalar::Util 'blessed';
+use Moose::Util ();
 
 sub mk_classdata {
   my ($class, $attribute) = @_;
@@ -13,7 +12,8 @@ sub mk_classdata {
   my $slot = '$'.$attribute;
   my $accessor =  sub {
     my $pkg = ref $_[0] || $_[0];
-    my $meta = $pkg->Class::MOP::Object::meta();
+    my $meta = Moose::Util::find_meta($pkg) 
+        || Moose->init_meta( for_class => $pkg );
     if (@_ > 1) {
       $meta->namespace->{$attribute} = \$_[1];
       return $_[1];
@@ -44,17 +44,18 @@ sub mk_classdata {
     unless ref $accessor eq 'CODE';
 
   my $meta = $class->Class::MOP::Object::meta();
-  my $immutable_options;
-  if( $meta->is_immutable ){
-    $immutable_options = $meta->get_immutable_options;
-    $meta->make_mutable;
-  }
+  confess "${class}'s metaclass is not a Class::MOP::Class"
+    unless $meta->isa('Class::MOP::Class');
+
+  my $was_immutable = $meta->is_immutable;
+  $meta->make_mutable if $was_immutable;
+
   my $alias = "_${attribute}_accessor";
   $meta->add_method($alias, $accessor);
   $meta->add_method($attribute, $accessor);
-  if(defined $immutable_options){
-    $meta->make_immutable(%{ $immutable_options });
-  }
+
+  $meta->make_immutable if $was_immutable;
+
   $class->$attribute($_[2]) if(@_ > 2);
   return $accessor;
 }
