@@ -1,18 +1,16 @@
 package Catalyst::Component;
 
-use strict;
-use base qw/Class::Accessor::Fast Class::Data::Inheritable/;
-use NEXT;
+use Moose;
+use Class::MOP;
+use Class::MOP::Object;
 use Catalyst::Utils;
+use Class::C3::Adopt::NEXT;
+use MRO::Compat;
+use mro 'c3';
+
+with 'MooseX::Emulate::Class::Accessor::Fast';
+with 'Catalyst::ClassData';
 
-BEGIN {
-    if (eval 'require Moose; 1') {
-        *__HAVE_MOOSE = sub () { 1 };
-    }
-    else {
-        *__HAVE_MOOSE = sub () { 0 };
-    }
-}
 
 =head1 NAME
 
@@ -57,33 +55,18 @@ component loader with config() support and a process() method placeholder.
 
 =cut
 
-__PACKAGE__->mk_classdata($_) for qw/_config _plugins/;
-
-
-
-sub new {
-    my ( $class, $c ) = @_;
+__PACKAGE__->mk_classdata('_plugins');
+__PACKAGE__->mk_classdata('_config');
 
+sub BUILDARGS {
+    my ($self) = @_;
+    
     # Temporary fix, some components does not pass context to constructor
     my $arguments = ( ref( $_[-1] ) eq 'HASH' ) ? $_[-1] : {};
 
-    my $config = $class->merge_config_hashes( $class->config, $arguments );
-
-    my $self = $class->NEXT::new($config);
-
-    if (__HAVE_MOOSE) {
-        my $meta = Class::MOP::get_metaclass_by_name($class);
-        if ($meta) {
-            $self = $meta->new_object(
-                __INSTANCE__ => $self,
-                %$config
-            );
-            # May not inherit from Moose::Object at all, so
-            # call BUILDALL explicitly.
-            $self->Moose::Object::BUILDALL($config);
-        }
-    }
-    return $self;
+    my $args =  $self->merge_config_hashes( $self->config, $arguments );
+    
+    return $args;
 }
 
 sub COMPONENT {
@@ -91,27 +74,20 @@ sub COMPONENT {
 
     # Temporary fix, some components does not pass context to constructor
     my $arguments = ( ref( $_[-1] ) eq 'HASH' ) ? $_[-1] : {};
-
-    if ( my $new = $self->NEXT::COMPONENT( $c, $arguments ) ) {
-        return $new;
-    }
-    else {
-        if ( my $new = $self->new( $c, $arguments ) ) {
-            return $new;
-        }
-        else {
-            my $class = ref $self || $self;
-            my $new   = $self->merge_config_hashes( 
-                $self->config, $arguments );
-            return bless $new, $class;
-        }
+    if( my $next = $self->next::can ){
+      my $class = blessed $self || $self;
+      my ($next_package) = Class::MOP::get_code_info($next);
+      warn "There is a COMPONENT method resolving after Catalyst::Component in ${next_package}.\n";
+      warn "This behavior can no longer be supported, and so your application is probably broken.\n";
+      warn "Your linearised isa hierarchy is: " . join(', ', mro::get_linear_isa($class)) . "\n";
+      warn "Please see perldoc Catalyst::Upgrading for more information about this issue.\n";
     }
+    return $self->new($c, $arguments);
 }
 
 sub config {
     my $self = shift;
-    my $config_sub = $self->can('_config');
-    my $config = $self->$config_sub() || {};
+    my $config = $self->_config || {};
     if (@_) {
         my $newconfig = { %{@_ > 1 ? {@_} : $_[0]} };
         $self->_config(
@@ -120,18 +96,13 @@ sub config {
     } else {
         # this is a bit of a kludge, required to make
         # __PACKAGE__->config->{foo} = 'bar';
-        # work in a subclass. Calling the Class::Data::Inheritable setter
-        # will create a new _config method in the current class if it's
-        # currently inherited from the superclass. So, the can() call will
-        # return a different subref in that case and that means we know to
-        # copy and reset the value stored in the class data.
-
-        $self->_config( $config );
-
-        if ((my $config_sub_now = $self->can('_config')) ne $config_sub) {
+        # work in a subclass.
+        my $class = blessed($self) || $self;
+        my $meta = Class::MOP::get_metaclass_by_name($class);
+        unless ($meta->has_package_symbol('$_config')) {
 
             $config = $self->merge_config_hashes( $config, {} );
-            $self->$config_sub_now( $config );
+            $self->_config( $config );
         }
     }
     return $config;
@@ -149,6 +120,9 @@ sub process {
           . " did not override Catalyst::Component::process" );
 }
 
+no Moose;
+
+__PACKAGE__->meta->make_immutable;
 1;
 
 __END__
@@ -196,7 +170,7 @@ Alias for the method in L<Catalyst::Utils>.
 
 =head2 ACCEPT_CONTEXT($c, @args)
 
-Catalyst components are normally initalized during server startup, either
+Catalyst components are normally initialized during server startup, either
 as a Class or a Instance. However, some components require information about
 the current request. To do so, they can implement an ACCEPT_CONTEXT method.
 
