@@ -1,7 +1,8 @@
 package Catalyst::DispatchType::Index;
 
-use strict;
-use base qw/Catalyst::DispatchType/;
+use Moose;
+extends 'Catalyst::DispatchType';
+no Moose;
 
 =head1 NAME
 
@@ -9,10 +10,21 @@ Catalyst::DispatchType::Index - Index DispatchType
 
 =head1 SYNOPSIS
 
-See L<Catalyst>.
+See L<Catalyst::DispatchType>.
 
 =head1 DESCRIPTION
 
+Dispatch type managing behaviour for index pages.  For more information on
+dispatch types, see:
+
+=over 4
+
+=item * L<Catalyst::Manual::Intro> for how they affect application authors
+
+=item * L<Catalyst::DispatchType> for implementation information.
+
+=back
+
 =head1 METHODS
 
 =head2 $self->match( $c, $path )
@@ -55,9 +67,9 @@ sub uri_for_action {
     return "/".$action->namespace;
 }
 
-=head1 AUTHOR
+=head1 AUTHORS
 
-Sebastian Riedel, C<sri@cpan.org>
+Catalyst Contributors, see Catalyst.pm
 
 =head1 COPYRIGHT
 
@@ -66,4 +78,6 @@ the same terms as Perl itself.
 
 =cut
 
+__PACKAGE__->meta->make_immutable;
+
 1;
