@@ -1,7 +1,7 @@
 package TestAppOnDemand::Controller::Body;
 
 use strict;
-use base 'Catalyst::Base';
+use base 'Catalyst::Controller';
 
 use Data::Dump ();
 
@@ -26,4 +26,4 @@ sub read : Local {
     $c->res->body( join ( '|', map { length $_ } @chunks ) );
 }
 
-1;
\ No newline at end of file
+1;
